@@ -18,6 +18,6 @@ def suite():
 
 
 if __name__ == '__main__':
-    runner = unittest.runner.TextTestRunner()
+    runner = unittest.runner.TextTestRunner(verbosity=2)
     result = runner.run(suite())
     sys.exit(not result.wasSuccessful())
